/*
 * This file is part of WCX_SFF. A Windows Commander plugin for SFF files.
 * Copyright (C) 2001  Amenna Heratsini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * email: amennah@hotmail.com
 */

#ifndef __SFF_H__
#define __SFF_H__

#include <windows.h>

typedef void (*sff_progress_handler)(int percentage);
void sff_set_encode_handler(sff_progress_handler handler);
void sff_set_decode_handler(sff_progress_handler handler);

void SFFencode_init(HANDLE handle);
void SFFencode(HANDLE handle, BYTE *image, WORD width, WORD height, WORD bpp);
void SFFencode_deinit(HANDLE handle);
// info:   write compressed 1-bit Structured Fax File
// input:  handle  = output file
//         image   = (width x height) 3-byte RGB values
//         width   = width of image
//         height  = height of image
//         bpp     = bits per pixel (set to 1)

void SFFdecode(int page, HANDLE handle, BYTE **image);
// info:   read 1 bit Structured Fax File
// input:  handle = input file
// output: image  = (width x height) 3-byte RGB values


int SFFinfo(int page, HANDLE handle, WORD *width, WORD *height, WORD *bpp);
// input:  page   = page number (0 = receive page numbers)
//         handle = input file
// output: width  = width of image
//         height = height of image
//         bpp    = bits per pixel (always 1)
// return:
//         if page==0 {
//             -255      = error
//             otherwise = page numbers
//         } else {
//             -255      = error
//             -254      = invalid page number
//             -253      = there was an error reading a page
//             -252      = unsupported coding scheme
//             <sign> 1  = Modified Huffman
//             where: <sign> = - if the file type is unsupported
//                             + if the file can be read
//         }

int SFFmoreinfo(WORD page, HANDLE handle, WORD *width, WORD *height, WORD *bpp, WORD *dpi_x, WORD *dpi_y);
// output: dpi_x = horizontal resolution
//         dpi_y = vertical resolution

#endif
