/*
	"unlzx.wcx" - EntpackerDLL fr (Amiga)LZX-Archive
	-> Code basiert auf dem Programm "unlzx.c" von ???


	17.12.2001	0.1	- erste Testversion

	12.01.2002	0.2	- Bugfix (Unterzeichnisse in den Archiven wurden nicht angelegt)
					- ProgressMonitor eingebaut (Code von Christian Ghisler bernommen)

	03.03.2002	0.3	- Bugfix (manche Archive wurden nicht richtig entpackt)

*/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "wcxhead.h"
#include "util.hpp"

// ----------------------------------------------------------------------

typedef struct t_FileInfoList
{
	char name[256];
	char pathname[256];
	unsigned int packsize;
	unsigned int unpacksize;
	unsigned int year;
	unsigned int month;
	unsigned int day;
	unsigned int hh24;
	unsigned int minute;
	unsigned int second;
	t_FileInfoList *prev;
	t_FileInfoList *next;
} t_FileInfoList;


typedef struct t_ArchivInfo
{
	char name[256];
	t_FileInfoList *files;
	tChangeVolProc ChangeVolProc;
	tProcessDataProc ProcessDataProc;
} t_ArchivInfo;

// ----------------------------------------------------------------------/

unsigned char info_header[10];
unsigned char archive_header[31];
unsigned char header_filename[256];
unsigned char header_comment[256];

unsigned int pack_size;
unsigned int unpack_size;

unsigned int crc;
unsigned int year, month, day;
unsigned int hour, minute, second;
unsigned char attributes;
unsigned char pack_mode;

/* ---------------------------------------------------------------------- */

struct filename_node
{
	struct filename_node *next;
	unsigned int length;
	unsigned int crc;
	char filename[256];
};

struct filename_node *filename_list;

/* ---------------------------------------------------------------------- */

unsigned char read_buffer[16384]; /* have a reasonable sized read buffer */
unsigned char decrunch_buffer[258+65536+258]; /* allow overrun for speed */

unsigned char *source;
unsigned char *destination;
unsigned char *source_end;
unsigned char *destination_end;

unsigned int decrunch_method;
unsigned int decrunch_length;
unsigned int last_offset;
unsigned int global_control;
int global_shift;

unsigned char offset_len[8];
unsigned short offset_table[128];
unsigned char huffman20_len[20];
unsigned short huffman20_table[96];
unsigned char literal_len[768];
unsigned short literal_table[5120];

/* ---------------------------------------------------------------------- */

static const unsigned char table_one[32]=
{
	0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14
};

static const unsigned int table_two[32]=
{
	0,1,2,3,4,6,8,12,16,24,32,48,64,96,128,192,256,384,512,768,1024,
	1536,2048,3072,4096,6144,8192,12288,16384,24576,32768,49152
};

static const unsigned int table_three[16]=
{
	0,1,3,7,15,31,63,127,255,511,1023,2047,4095,8191,16383,32767
};

static const unsigned char table_four[34]=
{
	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16
};

/* ---------------------------------------------------------------------- */

/* Read and build the decrunch tables. There better be enough data in the */
/* source buffer or it's stuffed. */

int read_literal_table()
{
unsigned int control;
int shift;
unsigned int temp; /* could be a register */
unsigned int symbol, pos, count, fix, max_symbol;
int abort = 0;

	control = global_control;
	shift = global_shift;

	if (shift < 0) /* fix the control word if necessary */
	{
		shift += 16;
		control += *source++ << (8 + shift);
		control += *source++ << shift;
	}

	/* read the decrunch method */

	decrunch_method = control & 7;
	control >>= 3;
	if((shift -= 3) < 0)
	{
		shift += 16;
		control += *source++ << (8 + shift);
		control += *source++ << shift;
	}

	/* Read and build the offset huffman table */

	if((!abort) && (decrunch_method == 3))
	{
		for(temp = 0; temp < 8; temp++)
		{
			offset_len[temp] = control & 7;
			control >>= 3;
			if((shift -= 3) < 0)
			{
				shift += 16;
				control += *source++ << (8 + shift);
				control += *source++ << shift;
			}
		}
		abort = make_decode_table(8, 7, offset_len, offset_table);
	}

	/* read decrunch length */

	if(!abort)
	{
		decrunch_length = (control & 255) << 16;
		control >>= 8;
		if((shift -= 8) < 0)
		{
			shift += 16;
			control += *source++ << (8 + shift);
			control += *source++ << shift;
		}
		decrunch_length += (control & 255) << 8;
		control >>= 8;
		if((shift -= 8) < 0)
		{
			shift += 16;
			control += *source++ << (8 + shift);
			control += *source++ << shift;
		}
		decrunch_length += (control & 255);
		control >>= 8;
		if((shift -= 8) < 0)
		{
			shift += 16;
			control += *source++ << (8 + shift);
			control += *source++ << shift;
		}
	}

	/* read and build the huffman literal table */

	if((!abort) && (decrunch_method != 1))
	{
		pos = 0;
		fix = 1;
		max_symbol = 256;

		do
		{
			for(temp = 0; temp < 20; temp++)
			{
				huffman20_len[temp] = control & 15;
				control >>= 4;
				if((shift -= 4) < 0)
				{
					shift += 16;
					control += *source++ << (8 + shift);
					control += *source++ << shift;
				}
			}
			abort = make_decode_table(20, 6, huffman20_len, huffman20_table);

			if(abort) break; /* argh! table is corrupt! */

			do
			{
				if((symbol = huffman20_table[control & 63]) >= 20)
				{
					do /* symbol is longer than 6 bits */
					{
						symbol = huffman20_table[((control >> 6) & 1) + (symbol << 1)];
						if(!shift--)
						{
							shift += 16;
							control += *source++ << 24;
							control += *source++ << 16;
						}
						control >>= 1;
					} while(symbol >= 20);
					temp = 6;
				}
				else
				{
					temp = huffman20_len[symbol];
				}
				control >>= temp;
				if((shift -= temp) < 0)
				{
					shift += 16;
					control += *source++ << (8 + shift);
					control += *source++ << shift;
				}
				switch(symbol)
				{
				case 17:
				case 18:
					{
						if(symbol == 17)
						{
							temp = 4;
							count = 3;
						}
						else /* symbol == 18 */
						{
							temp = 6 - fix;
							count = 19;
						}
						count += (control & table_three[temp]) + fix;
						control >>= temp;
						if((shift -= temp) < 0)
						{
							shift += 16;
							control += *source++ << (8 + shift);
							control += *source++ << shift;
						}
						while((pos < max_symbol) && (count--))
							literal_len[pos++] = 0;
						break;
					}
				case 19:
					{
						count = (control & 1) + 3 + fix;
						if(!shift--)
						{
							shift += 16;
							control += *source++ << 24;
							control += *source++ << 16;
						}
						control >>= 1;
						if((symbol = huffman20_table[control & 63]) >= 20)
						{
							do /* symbol is longer than 6 bits */
							{
								symbol = huffman20_table[((control >> 6) & 1) + (symbol << 1)];
								if(!shift--)
								{
									shift += 16;
									control += *source++ << 24;
									control += *source++ << 16;
								}
								control >>= 1;
							} while(symbol >= 20);
							temp = 6;
						}
						else
						{
							temp = huffman20_len[symbol];
						}
						control >>= temp;
						if((shift -= temp) < 0)
						{
							shift += 16;
							control += *source++ << (8 + shift);
							control += *source++ << shift;
						}
						symbol = table_four[literal_len[pos] + 17 - symbol];
						while((pos < max_symbol) && (count--))
							literal_len[pos++] = symbol;
						break;
					}
				default:
					{
						symbol = table_four[literal_len[pos] + 17 - symbol];
						literal_len[pos++] = symbol;
						break;
					}
				}
			} while(pos < max_symbol);
			fix--;
			max_symbol += 512;
		} while(max_symbol == 768);

		if(!abort)
			abort = make_decode_table(768, 12, literal_len, literal_table);
	}

	global_control = control;
	global_shift = shift;

	return(abort);
}


/* ---------------------------------------------------------------------- */

/* Fill up the decrunch buffer. Needs lots of overrun for both destination */
/* and source buffers. Most of the time is spent in this routine so it's  */
/* pretty damn optimized. */

void decrunch()
{
unsigned int control;
int shift;
unsigned int temp; /* could be a register */
unsigned int symbol, count;
unsigned char *string;

	control = global_control;
	shift = global_shift;

	do
	{
		if((symbol = literal_table[control & 4095]) >= 768)
		{
			control >>= 12;
			if((shift -= 12) < 0)
			{
				shift += 16;
				control += *source++ << (8 + shift);
				control += *source++ << shift;
			}
			do /* literal is longer than 12 bits */
			{
				symbol = literal_table[(control & 1) + (symbol << 1)];
				if(!shift--)
				{
					shift += 16;
					control += *source++ << 24;
					control += *source++ << 16;
				}
				control >>= 1;
			} while(symbol >= 768);
		}
		else
		{
			temp = literal_len[symbol];
			control >>= temp;
			if((shift -= temp) < 0)
			{
				shift += 16;
				control += *source++ << (8 + shift);
				control += *source++ << shift;
			}
		}
		if(symbol < 256)
		{
			*destination++ = symbol;
		}
		else
		{
			symbol -= 256;
			count = table_two[temp = symbol & 31];
			temp = table_one[temp];
			if((temp >= 3) && (decrunch_method == 3))
			{
				temp -= 3;
				count += ((control & table_three[temp]) << 3);
				control >>= temp;
				if((shift -= temp) < 0)
				{
					shift += 16;
					control += *source++ << (8 + shift);
					control += *source++ << shift;
				}
				count += (temp = offset_table[control & 127]);
				temp = offset_len[temp];
			}
			else
			{
				count += control & table_three[temp];
				if(!count) count = last_offset;
			}
			control >>= temp;
			if((shift -= temp) < 0)
			{
				shift += 16;
				control += *source++ << (8 + shift);
				control += *source++ << shift;
			}
			last_offset = count;

			count = table_two[temp = (symbol >> 5) & 15] + 3;
			temp = table_one[temp];
			count += (control & table_three[temp]);
			control >>= temp;
			if((shift -= temp) < 0)
			{
				shift += 16;
				control += *source++ << (8 + shift);
				control += *source++ << shift;
			}
			string = (decrunch_buffer + last_offset < destination) ? destination - last_offset : destination + 65536 - last_offset;
			do
			{
				*destination++ = *string++;
			} while(--count);
		}
	} while((destination < destination_end) && (source < source_end));

	global_control = control;
	global_shift = shift;
}



/* ---------------------------------------------------------------------- */

/* Trying to understand this function is hazardous. */

int extract_normal(FILE *in_file, t_ArchivInfo *archivinfo)
{
struct filename_node *node;
FILE *out_file = 0;
unsigned char *pos;
unsigned char *temp;
unsigned int sum;
unsigned int count;
int abort = 0;
bool depack;			// true, wenn wir gerade das aktuelle File am Wickel haben


	global_control = 0; // initial control word
	global_shift = -16;
	last_offset = 1;
	unpack_size = 0;
	decrunch_length = 0;

	for(count = 0; count < 8; count++)
		offset_len[count] = 0;
	for(count = 0; count < 768; count ++)
		literal_len[count] = 0;

	source_end = (source = read_buffer + 16384) - 1024;
	pos = destination_end = destination = decrunch_buffer + 258 + 65536;

	for(node = filename_list; (!abort) && node; node = node->next)
	{

		depack = strcmp(node->filename, archivinfo->files->name) == 0;	// nur das gewnschte File entpacken

		//printf("Extracting \"%s\"...", node->filename);
		//fflush(stdout);

		//out_file = open_output(node->filename);
		if (depack)
			out_file = open_output(archivinfo->files->pathname);
		else
			out_file = 0;

		sum = 0; /* reset CRC */

		unpack_size = node->length;

		while(unpack_size > 0)
		{

			if(pos == destination) /* time to fill the buffer? */
			{
				/* check if we have enough data and read some if not */
				if(source >= source_end) /* have we exhausted the current read buffer? */
				{
					temp = read_buffer;
					if(count = temp - source + 16384)
					{
						do /* copy the remaining overrun to the start of the buffer */
						{
							*temp++ = *source++;
						} while(--count);
					}
					source = read_buffer;
					count = source - temp + 16384;

					if (archivinfo->ProcessDataProc)
						if (archivinfo->ProcessDataProc(NULL, count)==0)
							abort=1;

					if (pack_size < count)
						count = pack_size; /* make sure we don't read too much */

					if (fread(temp, 1, count, in_file) != count)
					{
						/*
						printf("\n");
						if(ferror(in_file))
							perror("FRead(Data)");
						else
							fprintf(stderr, "EOF: Data\n");
						*/
						abort = 1;
						break; /* fatal error */
					}
					pack_size -= count;

					temp += count;
					if(source >= temp) break; /* argh! no more data! */
				} /* if(source >= source_end) */

				
				// check if we need to read the tables 
				if(decrunch_length <= 0)
				{
					if(read_literal_table()) break; // argh! can't make huffman tables! 
				}

				// unpack some data 
				if(destination >= decrunch_buffer + 258 + 65536)
				{
					if(count = destination - decrunch_buffer - 65536)
					{
						temp = (destination = decrunch_buffer) + 65536;
						do // copy the overrun to the start of the buffer
						{
							*destination++ = *temp++;
						} while(--count);
					}
					pos = destination;
				}
				destination_end = destination + decrunch_length;
				if(destination_end > decrunch_buffer + 258 + 65536)
					destination_end = decrunch_buffer + 258 + 65536;
				temp = destination;

				decrunch();

				decrunch_length -= (destination - temp);
				
			}

			/* calculate amount of data we can use before we need to fill the buffer again */
			count = destination - pos;
			if(count > unpack_size) count = unpack_size; /* take only what we need */

			crc_calc(pos, count, sum);

			if (out_file) /* Write the data to the file */
			{
				if(fwrite(pos, 1, count, out_file) != count)
				{
					//perror("FWrite"); /* argh! write error */
					fclose(out_file);
					out_file = 0;
				}
			}
			unpack_size -= count;
			pos += count;
		}

		if(out_file)
		{
			fclose(out_file);
			//if (!abort) printf(" crc %s\n", (node->crc == sum) ? "good" : "bad");
		}
	} /* for */

	return abort;
}

/* ---------------------------------------------------------------------- */

/* This is less complex than extract_normal. Almost decipherable. */

int extract_store(FILE *in_file, t_ArchivInfo *archivinfo)
{
struct filename_node *node;
FILE *out_file;
unsigned int sum;
unsigned int count;
int abort = 0;

	for(node = filename_list; (!abort) && node; node = node->next)
	{
		if (strcmp(node->filename, archivinfo->files->name) != 0)	// nur das gewnscht File entpacken
			continue;
		//printf("Storing \"%s\"...", node->filename);
		fflush(stdout);

		//out_file = open_output(node->filename);
		out_file = open_output(archivinfo->files->pathname);

		sum = 0; /* reset CRC */

		unpack_size = node->length;
		if(unpack_size > pack_size) unpack_size = pack_size;

		while(unpack_size > 0)
		{
			count = (unpack_size > 16384) ? 16384 : unpack_size;

			if(fread(read_buffer, 1, count, in_file) != count)
			{
				printf("\n");
				if(ferror(in_file))
					perror("FRead(Data)");
				else
					fprintf(stderr, "EOF: Data\n");
				abort = 1;
				break; /* fatal error */
			}
			pack_size -= count;

			crc_calc(read_buffer, count, sum);

			if(out_file) /* Write the data to the file */
			{
				if(fwrite(read_buffer, 1, count, out_file) != count)
				{
					perror("FWrite"); /* argh! write error */
					fclose(out_file);
					out_file = 0;
				}
			}
			unpack_size -= count;
			if (archivinfo->ProcessDataProc)
                if (archivinfo->ProcessDataProc(NULL,count)==0)
					abort=1;
		}

		if(out_file)
		{
			fclose(out_file);
			//if (!abort) printf(" crc %s\n", (node->crc == sum) ? "good" : "bad");
		}
	} /* for */

	return(abort);
}

/* ---------------------------------------------------------------------- */
/* Easiest of the three. Just print the file(s) we didn't understand. */

int extract_unknown(FILE *in_file, t_ArchivInfo *archivinfo)
{
	return 0;		// knnen hier eh nix machen

struct filename_node *node;
int abort = 0;

	for(node = filename_list; node; node = node->next)
	{
		printf("Unknown \"%s\"\n", node->filename);
	}

	return(abort);
}


/* ---------------------------------------------------------------------- */

/* Read the archive and build a linked list of names. Merged files is     */
/* always assumed. Will fail if there is no memory for a node. Sigh.      */

int extract_archive(FILE *in_file, t_ArchivInfo *archivinfo)
{
unsigned int sum;
unsigned int temp;
struct filename_node **filename_next;
struct filename_node *node;
struct filename_node *temp_node;
unsigned int actual;
int abort;
int result = 1; /* assume an error */

	filename_list = 0; /* clear the list */
	filename_next = &filename_list;

	do
	{
		abort = 1; /* assume an error */
		actual = fread(archive_header, 1, 31, in_file);
		if(!ferror(in_file))
		{
			if(actual) /* 0 is normal and means EOF */
			{
				if(actual == 31)
				{
					sum = 0; /* reset CRC */
					crc = (archive_header[29] << 24) + (archive_header[28] << 16) + (archive_header[27] << 8) + archive_header[26]; /* header crc */
					archive_header[29] = 0; /* Must set the field to 0 before calculating the crc */
					archive_header[28] = 0;
					archive_header[27] = 0;
					archive_header[26] = 0;
					crc_calc(archive_header, 31, sum);
					temp = archive_header[30]; /* filename length */
					actual = fread(header_filename, 1, temp, in_file);
					if(!ferror(in_file))
					{
						if(actual == temp)
						{
							header_filename[temp] = 0;
							crc_calc(header_filename, temp, sum);
							temp = archive_header[14]; /* comment length */
							actual = fread(header_comment, 1, temp, in_file);
							if(!ferror(in_file))
							{
								if(actual == temp)
								{
									header_comment[temp] = 0;
									crc_calc(header_comment, temp, sum);
									if(sum == crc)
									{
										unpack_size = (archive_header[5] << 24) + (archive_header[4] << 16) + (archive_header[3] << 8) + archive_header[2]; /* unpack size */
										pack_size = (archive_header[9] << 24) + (archive_header[8] << 16) + (archive_header[7] << 8) + archive_header[6]; /* packed size */
										pack_mode = archive_header[11]; /* pack mode */
										crc = (archive_header[25] << 24) + (archive_header[24] << 16) + (archive_header[23] << 8) + archive_header[22]; /* data crc */

										if(node = (struct filename_node *)malloc(sizeof(struct filename_node))) /* allocate a filename node */
										{
											*filename_next = node; /* add this node to the list */
											filename_next = &(node->next);
											node->next = 0;
											node->length = unpack_size;
											node->crc = crc;
											for(temp = 0; node->filename[temp] = header_filename[temp]; temp++);

											if(pack_size)
											{
												switch(pack_mode)
												{
												case 0: /* store */
													{
														abort = extract_store(in_file, archivinfo);
														break;
													}
												case 2: /* normal */
													{
														abort = extract_normal(in_file, archivinfo);
														break;
													}
												default: /* unknown */
													{
														abort = extract_unknown(in_file, archivinfo);
														break;
													}
												}
												if(abort) break; /* a read error occured */

												temp_node = filename_list; /* free the list now */
												while(node = temp_node)
												{
													temp_node = node->next;
													free(node);
												}
												filename_list = 0; /* clear the list */
												filename_next = &filename_list;

												if(fseek(in_file, pack_size, SEEK_CUR))
												{
													perror("FSeek(Data)");
													break;
												}
											}
											else
											{
												abort = 0; /* continue */
											}
										}
										else
										{
											//fprintf(stderr, "MAlloc(Filename_node)\n");
										}
									}
									else
									{
										//fprintf(stderr, "CRC: Archive_Header\n");
									}
								}
								else
								{
									//fprintf(stderr, "EOF: Header_Comment\n");
								}
							}
							else
							{
								//perror("FRead(Header_Comment)");
							}
						}
						else
						{
							//fprintf(stderr, "EOF: Header_Filename\n");
						}
					}
					else
					{
						//perror("FRead(Header_Filename)");
					}
				}
				else
				{
					//fprintf(stderr, "EOF: Archive_Header\n");
				}
			}
			else
			{
				result = 0; /* normal termination */
			}
		}
		else
		{
			//perror("FRead(Archive_Header)");
		}
	} while(!abort);

	/* free the filename list in case an error occured */
	temp_node = filename_list;
	while(node = temp_node)
	{
		temp_node = node->next;
		free(node);
	}

	return(result);
}

/* ---------------------------------------------------------------------- */

/* List the contents of an archive in a nice formatted kinda way.         */

int view_archive(FILE *in_file, t_ArchivInfo *archivinfo)
{
unsigned int sum;
unsigned int temp;
unsigned int total_pack = 0;
unsigned int total_unpack = 0;
unsigned int total_files = 0;
unsigned int merge_size = 0;
unsigned int actual;
int abort;
int result = 1; // assume an error


	do
	{
		abort = 1; // assume an error
		actual = fread(archive_header, 1, 31, in_file);
		if(!ferror(in_file))
		{
			if(actual != 0) // 0 is normal and means EOF
			{
				if(actual == 31)
				{
					sum = 0; // reset CRC
					crc = (archive_header[29] << 24) + (archive_header[28] << 16) + (archive_header[27] << 8) + archive_header[26];
					archive_header[29] = 0; // Must set the field to 0 before calculating the crc
					archive_header[28] = 0;
					archive_header[27] = 0;
					archive_header[26] = 0;
					crc_calc(archive_header, 31, sum);
					temp = archive_header[30]; // filename length
					actual = fread(header_filename, 1, temp, in_file);

					if(!ferror(in_file))
					{
						if(actual == temp)
						{
							header_filename[temp] = 0;
							crc_calc(header_filename, temp, sum);
							temp = archive_header[14]; // comment length
							actual = fread(header_comment, 1, temp, in_file);
							if(!ferror(in_file))
							{
								if(actual == temp)
								{
									header_comment[temp] = 0;
									crc_calc(header_comment, temp, sum);
									if(sum == crc)
									{
										if (archivinfo->files == NULL)
										{
											archivinfo->files = new t_FileInfoList;
											archivinfo->files->prev = NULL;
											archivinfo->files->next = NULL;
										}
										else
										{
											archivinfo->files->next = new t_FileInfoList;											
											archivinfo->files->next->prev = archivinfo->files;
											archivinfo->files->next->next = NULL;
											archivinfo->files = archivinfo->files->next;
										}

										attributes = archive_header[0]; // file protection modes
										archivinfo->files->unpacksize = unpack_size = (archive_header[5] << 24) + (archive_header[4] << 16) + (archive_header[3] << 8) + archive_header[2]; // unpack size
										pack_size = (archive_header[9] << 24) + (archive_header[8] << 16) + (archive_header[7] << 8) + archive_header[6]; // packed size
										temp = (archive_header[18] << 24) + (archive_header[19] << 16) + (archive_header[20] << 8) + archive_header[21]; // date
										archivinfo->files->year = year = ((temp >> 17) & 63) + 1970;
										archivinfo->files->month = month = (temp >> 23) & 15;
										archivinfo->files->day = day = (temp >> 27) & 31;
										archivinfo->files->hh24 = hour = (temp >> 12) & 31;
										archivinfo->files->minute = minute = (temp >> 6) & 63;
										archivinfo->files->second = second = temp & 63;

										total_pack += pack_size;
										total_unpack += unpack_size;
										total_files++;
										merge_size += unpack_size;
								

										if(archive_header[12] & 1)
										{
											archivinfo->files->packsize = 0;
										}
										else
										{
											archivinfo->files->packsize = pack_size;
										}

										strcpy(archivinfo->files->name, (const char *) header_filename);

										if(pack_size) // seek past the packed data
										{
											merge_size = 0;
											if(!fseek(in_file, pack_size, SEEK_CUR))
											{
												abort = 0; // continue
											}
											else
											{
												//perror("FSeek()");
											}
										}
										else
										{
											abort = 0; // continue
										}
									}
									else
									{
										//fprintf(stderr, "CRC: Archive_Header\n");
									}
								}
								else
								{
									//fprintf(stderr, "EOF: Header_Comment\n");
								}
							}
							else
							{
								//perror("FRead(Header_Comment)");
							}
						}
						else
						{
							//fprintf(stderr, "EOF: Header_Filename\n");
						}
					}
					else
					{
						//perror("FRead(Header_Filename)");
					}
				}
				else
				{
					//fprintf(stderr, "EOF: Archive_Header\n");
				}
			}
			else
			{
				result = 0; // normal termination
			}
		}
		else
		{
			//perror("FRead(Archive_Header)");
		}
	} while (!abort);

	return result;
}

/* ---------------------------------------------------------------------- */

/* Process a single archive. */

int process_archive(char *filename, int process_mode, t_ArchivInfo *archivinfo)
{
int result = 1; /* assume an error */
FILE *in_file;
int actual;

	if (in_file = fopen(filename,"rb"))
	{
		actual = fread(info_header, 1, 10, in_file);
		if (!ferror(in_file))
		{
			if (actual == 10)
			{
				if ((info_header[0] == 76) && (info_header[1] == 90) && (info_header[2] == 88)) /* LZX */
				{
					switch (process_mode)
					{
					case 1: /* extract archive */
						{
							result = extract_archive(in_file, archivinfo);
							break;
						}
					case 2: /* view archive */
						{
							result = view_archive(in_file, archivinfo);
							break;
						}
					}
				}
				//else fprintf(stderr, "Info_Header: Bad ID\n");
			}
			//else fprintf(stderr, "EOF: Info_Header\n");
		}
		//else perror("FRead(Info_Header)");
		fclose(in_file);
	}
	//else perror("FOpen(Archive)");

	return result;
}



typedef t_ArchivInfo* HANDLE;

extern "C" __declspec(dllexport) HANDLE OpenArchive (tOpenArchiveData *ArchiveData)
{
const int VIEW_MODE = 2;
int erg;
t_FileInfoList *fil;
t_ArchivInfo *archivinfo = NULL;

	if (ArchiveData->OpenMode == PK_OM_LIST || ArchiveData->OpenMode == PK_OM_EXTRACT)
	{

		archivinfo = new t_ArchivInfo;
		if (archivinfo == NULL)
		{
			ArchiveData->OpenResult = E_NO_MEMORY;
			return NULL;
		}
		archivinfo->files = NULL;
		strcpy(archivinfo->name, ArchiveData->ArcName);

		erg = process_archive(ArchiveData->ArcName, VIEW_MODE, archivinfo);

		if (erg != 0)
		{
			ArchiveData->OpenResult = E_UNKNOWN_FORMAT;
			return NULL;
		}
		else
		{
			ArchiveData->OpenResult = 0;
			fil = archivinfo->files;

			// put final (dummy) entry into dll (double linked list)
			if (fil)
			{
				while (fil->next)		// move to end of dll
					fil = fil->next;

				fil->next = new t_FileInfoList;
				if (fil->next == NULL)
				{
					ArchiveData->OpenResult = E_NO_MEMORY;
					return NULL;
				}
				strcpy(fil->next->name, "end of dll");
				fil->next->prev = fil;
				fil->next->next = NULL;
				fil = fil->next;
			}
			else						// no element in list
			{
				fil = new t_FileInfoList;
				if (fil == NULL)
				{
					ArchiveData->OpenResult = E_NO_MEMORY;
					return NULL;
				}
				fil->next = NULL;
				fil->prev = NULL;
			}
			
			while (fil->prev)
				fil = fil->prev;	// move to beginning of dll

			archivinfo->files = fil;

			return archivinfo;
		}
	}
	else
	{
		ArchiveData->OpenResult = E_NOT_SUPPORTED;
		return NULL;
	}
}

extern "C" __declspec(dllexport) int ReadHeader (HANDLE hArcData, tHeaderData *HeaderData)
{
char *p;

	if (hArcData->files->next)
	{
		strcpy(HeaderData->FileName, hArcData->files->name);
		p=HeaderData->FileName;
		while (*p!='\0')		// bis Stringende durchgehen
		{
			if (*p=='/')		// Unterverzeichnis?
				*p='\\';		// umwandeln in Backslash damit SubDir richtig erkannt wird
			p++;
		}
		HeaderData->FileTime = (hArcData->files->year - 1980) << 25 | hArcData->files->month << 21 | hArcData->files->day << 16 | hArcData->files->hh24 << 11 | hArcData->files->minute << 5 | hArcData->files->second / 2;
		HeaderData->PackSize = hArcData->files->packsize;
		HeaderData->UnpSize = hArcData->files->unpacksize;

		return 0;
	}
	else
	{
		while (hArcData->files->prev)
			hArcData->files = hArcData->files->prev;
		return E_END_ARCHIVE;
	}
}

extern "C" __declspec(dllexport) int ProcessFile (HANDLE hArcData, int Operation, char *DestPath, char *DestName)
{
const int EXTRACT_MODE = 1;
int erg;

	if (Operation == PK_EXTRACT)
	{
		strcpy(hArcData->files->pathname, DestName);

		erg = process_archive(hArcData->name, EXTRACT_MODE, hArcData);

		if (erg != 0)
			return E_BAD_ARCHIVE;

		hArcData->files = hArcData->files->next;
		return 0;
	}
	else if (Operation == PK_TEST || Operation == PK_SKIP)
	{
		hArcData->files = hArcData->files->next;
		return 0;
	}
	else
		return E_NOT_SUPPORTED;
}

extern "C" __declspec(dllexport) int CloseArchive (HANDLE hArcData)
{

	// does the dll has at least one element?
	if (hArcData->files)
	{
		// go to end of dll
		while (hArcData->files->next)
			hArcData->files = hArcData->files->next;
		// remove dll
		while (hArcData->files)
		{
			// check for last element
			if (hArcData->files->prev)
			{
				hArcData->files = hArcData->files->prev;
				delete hArcData->files->next;
			}
			else
			{
				delete hArcData->files;
				hArcData->files = NULL;
			}
		}
	}

	
	delete hArcData;
	hArcData = NULL;

	return 0;
}

extern "C" __declspec(dllexport) void SetChangeVolProc (HANDLE hArcData, tChangeVolProc pChangeVolProc)
{
	hArcData->ChangeVolProc=pChangeVolProc;
}

extern "C" __declspec(dllexport) void SetProcessDataProc (HANDLE hArcData, tProcessDataProc pProcessDataProc)
{
	hArcData->ProcessDataProc=pProcessDataProc;
}
