/*
 * This file is part of WCX_SFF. A Windows Commander plugin for SFF files.
 * Copyright (C) 2001  Amenna Heratsini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * email: amennah@hotmail.com
 */

#include <windows.h>
#include <stdio.h>
#include "wcxhead.h"
#include "sff.h"
#include "bmp.h"
#include "time.h"

static char fname[260];
static int pages, first;
static WORD width, height;
static tProcessDataProc pProcessDataProc;
static tChangeVolProc pChangeVolProc;
static WORD stime, sdate;

int __stdcall GetPackerCaps()
{
    return(PK_CAPS_NEW|PK_CAPS_MULTIPLE);
}

int __stdcall PackFiles(char *PackedFile, char *SubPath, char *SrcPath, char *AddList, int Flags)
{
    char buf[256];
    WORD width, height, bpp;
    int ver;
    BYTE *bmpimage;
    HANDLE handle, h;
    char *p=AddList;

    handle=CreateFile(PackedFile, GENERIC_READ|GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
    if (handle==INVALID_HANDLE_VALUE) return(E_ECREATE);

    SFFencode_init(handle);

    while (p[0]!=0) {
        sprintf(buf,"%s%s", SrcPath, p);

        if (pProcessDataProc(buf, width*height/24)==0) {
            SFFencode_deinit(handle);
            CloseHandle(handle);
            return(E_EABORTED);
        }

        h=CreateFile(buf, GENERIC_READ,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
        if (h==INVALID_HANDLE_VALUE) return(E_EOPEN);
    
        ver=BMPinfo(h, &width, &height, &bpp);
    
        if (ver<0) {
            CloseHandle(h);
            return(E_BAD_DATA);
        }
            
        bmpimage=(BYTE *)malloc(width*height*3);
        if (!bmpimage) return(E_NO_MEMORY);
        BMPdecode(h,&bmpimage);
        CloseHandle(h);
    
        if (pProcessDataProc(buf, width*height/12)==0) {
            free(bmpimage);
            SFFencode_deinit(handle);
            CloseHandle(handle);
            return(E_EABORTED);
        }

        SFFencode(handle, bmpimage, width, height, 1);
    
        free(bmpimage);

        p+=strlen(p)+1;
    }

    SFFencode_deinit(handle);

    CloseHandle(handle);

    return(0);
}

int __stdcall DeleteFiles(char *PackedFile, char *DeleteList)
{
    return(0);
}

void __stdcall ConfigurePacker(HWND Parent, HINSTANCE DllInstance)
{
}

HANDLE __stdcall OpenArchive(tOpenArchiveData *ArchiveData)
{
    FILETIME ft;
    HANDLE handle;

    handle=CreateFile(ArchiveData->ArcName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
    if (handle==INVALID_HANDLE_VALUE) { ArchiveData->OpenResult=E_EOPEN; return(0); }

    GetFileTime(handle, NULL, NULL, &ft);
    FileTimeToDosDateTime(&ft, &sdate, &stime);

    first=-1;

    switch (ArchiveData->OpenMode) {
        case PK_OM_LIST:    // open for filename reading
        case PK_OM_EXTRACT: // open for processing
           pages=SFFinfo(0, handle, NULL, NULL, NULL);
           if (pages<0) {
               ArchiveData->OpenResult=E_UNKNOWN_FORMAT;
               CloseHandle(handle);
               return(0);
           }

           break;
    }      

    strcpy(fname,ArchiveData->ArcName);

    return(handle);
}

int WriteImage(HANDLE handle, char *DestName)
{
    HANDLE h;
    int i;
    char nr[4];

    BYTE *sffimage=(BYTE *)calloc(width*height,3);
    if (!sffimage) return(E_NO_MEMORY);

    nr[0]=DestName[strlen(DestName)-7];
    nr[1]=DestName[strlen(DestName)-6];
    nr[2]=DestName[strlen(DestName)-5];
    nr[3]=0;
    i=atoi(nr);

    if (pProcessDataProc(NULL, width*height*1)==0) {
        free(sffimage);
        return(E_EABORTED);
    }
    SFFdecode(i, handle, &sffimage);
    if (pProcessDataProc(NULL, width*height*2)==0) {
        free(sffimage);
        return(E_EABORTED);
    }

    h=CreateFile(DestName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
    if (h==INVALID_HANDLE_VALUE) return(E_ECREATE);

    BMPencode(h,sffimage,width,height,1);

    if (!CloseHandle(h)) return(E_ECLOSE);
    free(sffimage);

    return(0);
}

int __stdcall ReadHeader(HANDLE hArcData, tHeaderData *HeaderData)
{
    char nam[260];

    if (first==0) return(E_END_ARCHIVE);

    if (first==-1) first=pages;

    if (first>0) {
        WORD bytesperline;
        int ver=SFFmoreinfo(first, hArcData, &width, &height, NULL, NULL, NULL);
        if (ver<0) return(E_EREAD);

        strcpy(nam, strrchr(fname,'\\'));
        nam[strlen(nam)-4]=0;
        sprintf(nam,"%s %03d.bmp", nam,first);
        strcpy(HeaderData->FileName,&nam[1]);
        HeaderData->PackSize=0;

        bytesperline=width/8;
        if (width%8) bytesperline++;

        HeaderData->UnpSize=bytesperline*height+sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+4*2;
        HeaderData->FileTime=sdate<<16 | stime;
        HeaderData->FileAttr=0x20;

        first--;
    }

    return(0);
}

int __stdcall ProcessFile(HANDLE hArcData, int Operation, char *DestPath, char *DestName)
{
    switch (Operation) {
        case PK_SKIP:
        case PK_TEST:
            return(0);

        case PK_EXTRACT:
            return(WriteImage(hArcData, DestName));
            break;
    }
    
    return(0);
}

int __stdcall CloseArchive(HANDLE hArcData)
{
    if (!CloseHandle(hArcData)) return(E_ECLOSE);

    return(0);
}

void __stdcall SetChangeVolProc(HANDLE hArcData, tChangeVolProc pChangeVolProc1)
{
    pChangeVolProc=pChangeVolProc1;
}

void __stdcall SetProcessDataProc(HANDLE hArcData, tProcessDataProc pProcessDataProc1)
{
    pProcessDataProc=pProcessDataProc1;
}

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    return(TRUE);
}
